#!/usr/bin/python

'''
Copyright (c) 2016, Jan Razlaw
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''

import sys
import os
import rosbag

# get name of bagfile
bagname = sys.argv[1]
tmp = bagname.split('.')
bagname = str(tmp[0])
print bagname
outname = ' '
count = 0

statusTopic = "/fcu/fcu_status"
logTopic = "/mod_log_note"

# 0 for standing ; 1 for flight
status = 0
statusString = "_standing"
currenttime = 0
currentStamp = 0
logEnabled = 0

# check if status is standing or flying at the beginning of the bag file and adjust parameters accordingly
with rosbag.Bag(bagname + ".bag") as initbag:
    for topic, msg, t in initbag.read_messages():
        # This also replaces tf timestamps under the assumption 
        if topic == "/fcu/fcu_status" and msg.motor_on == 0:
            outname = bagname + "_" + str(count) + "_standing" + ".bag"
            status = 0
            currenttime = initbag.get_start_time()
            break
         
        if topic == "/fcu/fcu_status" and msg.motor_on == 1:
            outname = bagname + "_" + str(count) + "_flight" + ".bag"
            status = 1
            currenttime = initbag.get_start_time()
            break
    initbag.close()

print outname  

# open a file for log messages from logTopic 
f = open(bagname + "_log", 'w')
f.write('time          msgs\n')

# split the input bagfile in parts wrt. status
with rosbag.Bag(bagname + ".bag") as inputbag:
    with rosbag.Bag(outname, 'w') as outbag:
        for topic, msg, t in inputbag.read_messages():
            # attempt to fix the bug that sets the start time of the bag to zero
            if msg._has_header:
                currenttime = msg.header.stamp.to_sec()
                currentStamp = msg.header.stamp
            # if status topic switches, split bag 
            if topic == statusTopic and msg.motor_on != status:
                if status == 0:
                    status = 1
                    statusString = "_flight"
                else:
                    status = 0
                    statusString = "_standing"
                outbag.close()
                
                print outname
                # reindex (necessary for unknown reasons)               
                outbag.reindex()
                
                # open new bag file for next part 
                count = count + 1 
                outname = bagname + "_" + str(count) + statusString + ".bag"
                outbag._open_write(outname)
                print outname

            # store msg in current bag part 
            outbag.write(topic, msg, t)
            
            # if logTopic store contained message and time in a file 
            if topic == logTopic:
                logmsgs = str(msg.data)
                logtime = currenttime - inputbag.get_start_time()
                s = str(str(logtime) + "  " + logmsgs + "\n")
                f.write(s)
                logEnabled = 1 
                    
f.close()

# delete empty log file if no log messages were contained 
if logEnabled == 0:
    os.remove(bagname + "_log")
